@@ -1,25 +1,29 @@
 # -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MIT
+from __future__ import absolute_import
 import errno
 import os
-import tempfile
 import shutil
+import tempfile
 
 import dnf
 import kobo.rpmlib
 import koji
 import six.moves.xmlrpc_client as xmlrpclib
 
-from module_build_service import conf, log, models, Modulemd, scm
-from module_build_service.builder.KojiModuleBuilder import (
-    koji_retrying_multicall_map, KojiModuleBuilder,
+from module_build_service.common import conf, log, models, scm
+from module_build_service.common.errors import UnprocessableEntity
+from module_build_service.common.koji import get_session, koji_retrying_multicall_map
+from module_build_service.common.modulemd import Modulemd
+from module_build_service.common.resolve import (
+    expand_single_mse_streams, get_compatible_base_module_mmds
 )
-from module_build_service.errors import UnprocessableEntity
 from module_build_service.resolver.base import GenericResolver
-from module_build_service.utils import retry
+from module_build_service.common.retry import retry
+from module_build_service.scheduler.db_session import db_session
 
 
-def add_default_modules(db_session, mmd, arches):
+def add_default_modules(mmd):
     """
     Add default modules as buildrequires to the input modulemd.
 
@@ -27,10 +31,7 @@ def add_default_modules(db_session, mmd, arches):
     a URL to a text file in xmd.mbs.default_modules_url. Any default module that isn't in the
     database will be logged and ignored.
 
-    :param db_session: a SQLAlchemy database session
     :param Modulemd.ModuleStream mmd: the modulemd of the module to add the module defaults to
-    :param list arches: the arches to limit the external repo queries to; this should be the arches
-        the module will be built with
     :raises RuntimeError: if the buildrequired base module isn't in the database or the default
         modules list can't be downloaded
     """
@@ -84,22 +85,52 @@ def add_default_modules(db_session, mmd, arches):
                 )
                 continue
 
-            try:
-                # We are reusing resolve_requires instead of directly querying the database since it
-                # provides the exact format that is needed for mbs.xmd.buildrequires.
-                #
-                # Only one default module is processed at a time in resolve_requires so that we
-                # are aware of which modules are not in the database, and can add those that are as
-                # buildrequires.
-                resolver = GenericResolver.create(db_session, conf)
-                resolved = resolver.resolve_requires([ns])
-            except UnprocessableEntity:
+            # Query for the latest default module that was built against this base module
+            resolver = GenericResolver.create(db_session, conf)
+            base_mmds = get_compatible_base_module_mmds(resolver, bm_mmd)
+            base_mmds = base_mmds["ready"] + base_mmds["garbage"]
+            base_mmds.sort(
+                key=lambda mmd: models.ModuleBuild.get_stream_version(mmd.get_stream_name(), False),
+                reverse=True)
+            for base_mmd in base_mmds:
+                default_module_mmds = resolver.get_buildrequired_modulemds(name, stream, base_mmd)
+                if not default_module_mmds:
+                    continue
+
+                # We need to ensure that module built against compatible base module stream
+                # really contains runtime-dependency on the current base module stream.
+                # For example in Fedora, we can have platform:f30 and platform:f31 base module
+                # streams. There can be foo:1 module built against platform:f30 which can work with
+                # any platform ("requires: platform: []"). This module can be configured as default
+                # module for platform:f28 and we need to support this case, but in the same time we
+                # cannot simply add any platform:f27 based module to platform:f28.
+                module_found = False
+                for default_module_mmd in default_module_mmds:
+                    for deps in default_module_mmd.get_dependencies():
+                        streams = deps.get_runtime_streams(module_name)
+                        if streams is None:
+                            continue
+                        streams = expand_single_mse_streams(db_session, module_name, streams)
+                        if bm_info["stream"] in streams:
+                            module_found = True
+                            break
+                    else:
+                        log.info(
+                            "Not using module %s as default module, because it does not "
+                            "contain runtime dependency on %s", default_module_mmd.get_nsvc(),
+                            bm_nsvc)
+                if module_found:
+                    break
+            else:
                 log.warning(
                     "The default module %s from %s is not in the database and couldn't be added as "
                     "a buildrequire",
                     ns, bm_nsvc,
                 )
                 continue
+            # Use resolve_requires since it provides the exact format that is needed for
+            # mbs.xmd.buildrequires
+            resolved = resolver.resolve_requires([default_module_mmd.get_nsvc()])
 
             nsvc = ":".join([name, stream, resolved[name]["version"], resolved[name]["context"]])
             log.info("Adding the default module %s as a buildrequire", nsvc)
@@ -120,7 +151,7 @@ def _get_default_modules(stream, default_modules_scm_url):
     :return: a dictionary where the keys are default module names and the values are default module
         streams
     :rtype: dict
-    :raise ValueError: if no default modules can be retrieved for that stream
+    :raise RuntimeError: if no default modules can be retrieved for that stream
     """
     scm_obj = scm.SCM(default_modules_scm_url)
     temp_dir = tempfile.mkdtemp()
@@ -166,7 +197,7 @@ def _get_default_modules(stream, default_modules_scm_url):
     except:  # noqa: E722
         msg = "Failed to retrieve the default modules"
         log.exception(msg)
-        raise ValueError(msg)
+        raise RuntimeError(msg)
     finally:
         shutil.rmtree(temp_dir)
 
@@ -179,7 +210,7 @@ def _get_rawhide_version():
     :return: the rawhide version (e.g. "f32")
     :rtype: str
     """
-    koji_session = KojiModuleBuilder.get_session(conf, login=False)
+    koji_session = get_session(conf, login=False)
     build_target = koji_session.getBuildTarget("rawhide")
     if build_target:
         return build_target["build_tag_name"].partition("-build")[0]
@@ -222,7 +253,7 @@ def handle_collisions_with_base_module_rpms(mmd, arches):
         "Querying Koji for the latest RPMs from the buildrequired base modules from the tags: %s",
         ", ".join(bm_tags),
     )
-    koji_session = KojiModuleBuilder.get_session(conf, login=False)
+    koji_session = get_session(conf, login=False)
     bm_rpms = _get_rpms_from_tags(koji_session, list(bm_tags), arches)
     # The keys are base module RPM names and the values are sets of RPM NEVRAs with that name
     name_to_nevras = {}
@@ -315,60 +346,72 @@ def _get_rpms_in_external_repo(repo_url, arches, cache_dir_name):
         )
 
     base = dnf.Base()
-    dnf_conf = base.conf
-    # Expire the metadata right away so that when a repo is loaded, it will always check to see if
-    # the external repo has been updated
-    dnf_conf.metadata_expire = 0
-
-    cache_location = os.path.join(conf.cache_dir, "dnf", cache_dir_name)
     try:
-        # exist_ok=True can't be used in Python 2
-        os.makedirs(cache_location, mode=0o0770)
-    except OSError as e:
-        # Don't fail if the directories already exist
-        if e.errno != errno.EEXIST:
-            log.exception("Failed to create the cache directory %s", cache_location)
-            raise RuntimeError("The MBS cache is not writeable.")
+        dnf_conf = base.conf
+        # Expire the metadata right away so that when a repo is loaded, it will always check to
+        # see if the external repo has been updated
+        dnf_conf.metadata_expire = 0
 
-    # Tell DNF to use the cache directory
-    dnf_conf.cachedir = cache_location
-    # Don't skip repos that can't be synchronized
-    dnf_conf.skip_if_unavailable = False
-    # Get rid of everything to be sure it's a blank slate. This doesn't delete the cached repo data.
-    base.reset(repos=True, goal=True, sack=True)
+        cache_location = os.path.join(conf.cache_dir, "dnf", cache_dir_name)
+        try:
+            # exist_ok=True can't be used in Python 2
+            os.makedirs(cache_location, mode=0o0770)
+        except OSError as e:
+            # Don't fail if the directories already exist
+            if e.errno != errno.EEXIST:
+                log.exception("Failed to create the cache directory %s", cache_location)
+                raise RuntimeError("The MBS cache is not writeable.")
 
-    # Add a separate repo for each architecture
-    for arch in arches:
-        # Convert arch to canon_arch. This handles cases where Koji "i686" arch is mapped to
-        # "i386" when generating RPM repository.
-        canon_arch = koji.canonArch(arch)
-        repo_name = "repo_{}".format(canon_arch)
-        repo_arch_url = repo_url.replace("$arch", canon_arch)
-        base.repos.add_new_repo(
-            repo_name, dnf_conf, baseurl=[repo_arch_url], minrate=conf.dnf_minrate,
-        )
+        # Tell DNF to use the cache directory
+        dnf_conf.cachedir = cache_location
+        # Don't skip repos that can't be synchronized
+        dnf_conf.skip_if_unavailable = False
+        dnf_conf.timeout = conf.dnf_timeout
+        # Get rid of everything to be sure it's a blank slate. This doesn't delete the cached repo
+        # data.
+        base.reset(repos=True, goal=True, sack=True)
 
-    try:
-        # Load the repos in parallel
-        base.update_cache()
-    except dnf.exceptions.RepoError:
-        msg = "Failed to load the external repos"
-        log.exception(msg)
-        raise RuntimeError(msg)
+        # Add a separate repo for each architecture
+        for arch in arches:
+            # Convert arch to canon_arch. This handles cases where Koji "i686" arch is mapped to
+            # "i386" when generating RPM repository.
+            canon_arch = koji.canonArch(arch)
+            repo_name = "repo_{}".format(canon_arch)
+            repo_arch_url = repo_url.replace("$arch", canon_arch)
+            base.repos.add_new_repo(
+                repo_name, dnf_conf, baseurl=[repo_arch_url], minrate=conf.dnf_minrate,
+            )
 
-    base.fill_sack(load_system_repo=False)
+        try:
+            # Load the repos in parallel
+            base.update_cache()
+        except dnf.exceptions.RepoError:
+            msg = "Failed to load the external repos"
+            log.exception(msg)
+            raise RuntimeError(msg)
 
-    # Return all the available RPMs
-    nevras = set()
-    for rpm in base.sack.query().available():
-        rpm_dict = {
-            "arch": rpm.arch,
-            "epoch": rpm.epoch,
-            "name": rpm.name,
-            "release": rpm.release,
-            "version": rpm.version,
-        }
-        nevra = kobo.rpmlib.make_nvra(rpm_dict, force_epoch=True)
-        nevras.add(nevra)
+        # dnf will not always raise an error on repo failures, so we check explicitly
+        for repo_name in base.repos:
+            if not base.repos[repo_name].metadata:
+                msg = "Failed to load metadata for repo %s" % repo_name
+                log.exception(msg)
+                raise RuntimeError(msg)
+
+        base.fill_sack(load_system_repo=False)
+
+        # Return all the available RPMs
+        nevras = set()
+        for rpm in base.sack.query().available():
+            rpm_dict = {
+                "arch": rpm.arch,
+                "epoch": rpm.epoch,
+                "name": rpm.name,
+                "release": rpm.release,
+                "version": rpm.version,
+            }
+            nevra = kobo.rpmlib.make_nvra(rpm_dict, force_epoch=True)
+            nevras.add(nevra)
+    finally:
+        base.close()
 
     return nevras
